package javacomp.basics;

import java.lang.annotation.*;
import java.lang.reflect.*;
import java.util.*;

@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.FIELD)
@interface Inject
{
}

interface Startable
{
    public void start();
}

class A1 implements Startable
{
    @Inject
    private A2 a2;
    
    @Inject
    private A3 a3;
    
    private String name = "A1Class";

    public String getName()
    {
        return name;
    }

    public void start()
    {
        System.out.println("A1.start:");
        if(a2 != null)
        {
            System.out.println("   a2 != null: " + a2.getName());
        }
        else
        {
            System.out.println("   a2 == null");
            
        }
        if(a3 != null)
        {
            System.out.println("   a3 != null: " + a3.getName());
        }
        else
        {
            System.out.println("   a3 == null");
            
        }
    }
}

class A2 implements Startable
{
    private String name = "A2Class";

    public String getName()
    {
        return name;
    }

    public void start()
    {
        System.out.println("A2.start:");
    }
}

class A3 implements Startable
{
    private String name = "A3Class";

    public String getName()
    {
        return name;
    }

    public void start()
    {
        System.out.println("A3.start:");
    }
}

public class DependencyInjection
{
    public static void main(String[] args)
    {
        if(args.length == 0)
        {
            System.out.println("arguments: list of class names");
            return;
        }
        HashMap<String,Object> map = new HashMap<String,Object>();
        createObjects(args, map);
        wireObjects(map);
        startObjects(map);
    }
    
    private static void createObjects(String[] args, HashMap<String,Object> map)
    {
        for(String className: args)
        {
            try
            {
                if(!map.containsKey(className))
                {
                    Class<?> c = Class.forName(className);
                    Object o = c.newInstance();
                    map.put(className, o);
                }
            }
            catch(Exception e)
            {
                System.out.println("Error in createObjects: " + 
                                   e.getMessage());
            }
        }
    }
    
    private static void wireObjects(HashMap<String,Object> map)
    {
        Collection<Object> objects = map.values();
        for(Object object: objects)
        {
            Class<?> c = object.getClass();
            Field[] attributes = c.getDeclaredFields();
            for(Field attribute: attributes)
            {
                if(attribute.isAnnotationPresent(Inject.class))
                {
                    String attClassName = attribute.getType().getName();
                    Object target = map.get(attClassName);
                    if(target != null)
                    {
                        attribute.setAccessible(true);
                        try
                        {
                            attribute.set(object, target);
                        }
                        catch(Exception e)
                        {
                            System.out.println("Error in wireObjects: " + 
                                               e.getMessage());
                        }
                    }
                }
            }
        }        
    }
    
    private static void startObjects(HashMap<String,Object> map)
    {
        Collection<Object> objects = map.values();
        for(Object object: objects)
        {
            if(object instanceof Startable)
            {
                Startable startableObject = (Startable) object;
                startableObject.start();
            }
        }
    }
}
