package javacomp.prototype.framework;

import java.io.*;

public class Main
{
    private static final String DEPLOYMENT_DIRECTORY = "/Users/oechsle/Desktop";
    private static final String WORKING_DIRECTORY = DEPLOYMENT_DIRECTORY +
                                                    "/work";
    private static final int SLEEPING_TIME = 15000;
    
    public static void main(String[] args)
    {
        CheckedDirectory deployDir = new CheckedDirectory(DEPLOYMENT_DIRECTORY);
        WorkSpace workSpace = new WorkSpace(WORKING_DIRECTORY);
        ComponentManager compManager = new ComponentManager();
        DeploymentDirectoryListener listener =
                        new DeploymentDirectoryListener(workSpace, compManager);
        deployDir.addDirectoryListener(listener);
        
        TerminatorThread tt = new TerminatorThread(Thread.currentThread());
        tt.start();
        try
        {
            while(!Thread.currentThread().isInterrupted())
            {
                System.out.println("---check deployment directory");
                deployDir.check();
                Thread.sleep(SLEEPING_TIME);
            }
        }
        catch (InterruptedException e)
        {
        }
        compManager.removeAllComponents();
        workSpace.clean();
    }
}

class TerminatorThread extends Thread
{
    private Thread mainThread;

    public TerminatorThread(Thread mainThread)
    {
        this.mainThread = mainThread;
    }
    
    public void run()
    {
        BufferedReader input = new BufferedReader(
                                   new InputStreamReader(
                                       System.in));
        try
        {
            input.readLine();
        }
        catch(IOException e)
        {
        }
        mainThread.interrupt();
    }
}
