package javacomp.osgi.bundle2;

import javacomp.osgi.bundle0.Counter;

import org.osgi.framework.*;

public class CounterClientActivator implements BundleActivator
{
    private ServiceReference<Counter> ref;
    private CounterThread thread;
    
    public void start(BundleContext context) throws Exception
    {
        System.out.println("CounterClientActivator.start");
        ref = context.getServiceReference(Counter.class);
        if(ref != null)
        {
            Counter counter = context.getService(ref);
            if(counter != null)
            {
                if(thread == null)
                {
                    System.out.println("using CounterImpl object created by server");
                    thread = new CounterThread(counter);
                    thread.start();                
                }
            }
            else
            {
                System.out.println("getService: null");
            }
        }
        else
        {
            System.out.println("getServiceReference: null");
        }
        /*
        Counter counter = new CounterImpl();
        System.out.println("CounterImpl object created by client");
        thread = new CounterThread(counter);
        thread.start();
        */
        
    }

    public void stop(BundleContext context) throws Exception
    {
        System.out.println("CounterClientActivator.stop");
        if(thread != null)
        {
            thread.interrupt();
            thread = null;
        }
        if(ref != null)
        {
            context.ungetService(ref);
            System.out.println("   with ungetService");
        }
    }
}

class CounterThread extends Thread
{
    private Counter counter;
    
    public CounterThread(Counter counter)
    {
        this.counter = counter;
    }
    
    public void run()
    {
        System.out.println("Thread: " + counter.getClass().getName());
        try
        {
            while(!isInterrupted())
            {
                String output = counter.version() + " (" +
                                counter.increment() + ")";
                System.out.println(output);
                sleep(2000);
            }
        }
        catch(InterruptedException e)
        {
        }
    }
}
