package javacomp.osgi.bundle3;

import javacomp.osgi.bundle0.*;
import org.osgi.framework.*;

public class CounterClientActivator implements BundleActivator, ServiceListener
{
    private BundleContext context;
    private CounterThread thread;
    
    public void start(BundleContext c) throws Exception
    {
        System.out.println("CounterClientActivator.start");
        this.context = c;
        startClient();
        c.addServiceListener(this);
    }

    public void stop(BundleContext c) throws Exception
    {
        System.out.println("CounterClientActivator.stop");
        stopClient(true);
        c.removeServiceListener(this);
    }

    public void serviceChanged(ServiceEvent event)
    {
        switch(event.getType())
        {
            case ServiceEvent.REGISTERED:
                System.out.println("CounterClientActivator.serviceChanged(REGISTERED)");
                startClient();
                break;
            case ServiceEvent.UNREGISTERING:
                System.out.println("CounterClientActivator.serviceChanged(UNREGISTERING)");
                stopClient(false);
                break;
            case ServiceEvent.MODIFIED:
                System.out.println("CounterClientActivator.serviceChanged(MODIFIED)");
                stopClient(false);
                startClient();
                break;
            default:
        }
    }
    
    private void startClient()
    {
        ServiceReference<Counter> ref = context.getServiceReference(Counter.class);
        if(ref != null)
        {
            Counter hello = (Counter) context.getService(ref);
            if(hello != null)
            {
                if(thread == null)
                {
                    thread = new CounterThread(hello);
                    thread.start();                
                }
            }
        }
    }
    
    private void stopClient(boolean uncoditionally)
    {
        ServiceReference<Counter> ref = context.getServiceReference(Counter.class);
        if(uncoditionally || ref == null)
        {
            if(thread != null)
            {
                thread.interrupt();
                try
                {
                    thread.join();
                }
                catch(InterruptedException e)
                {
                }
                thread = null;
            }
        }
    }
}

class CounterThread extends Thread
{
    private Counter counter;
    
    public CounterThread(Counter counter)
    {
        this.counter = counter;
    }
    
    public void run()
    {
        try
        {
            while(!isInterrupted())
            {
                String output = counter.version() + " (" +
                                counter.increment() + ")";
                System.out.println(output);
                sleep(2000);
            }
        }
        catch(InterruptedException e)
        {
        }
    }
}
