package javacomp.ejb.sleep.client;

import java.util.Properties;

import javacomp.ejb.sleep.server.Sleep;

import javax.naming.*;


public class SleepClient
{
    public static void main(String[] args) throws Exception
    {
        Properties properties = new Properties();
        properties.put(Context.INITIAL_CONTEXT_FACTORY,
                       "org.jnp.interfaces.NamingContextFactory");
        properties.put(Context.PROVIDER_URL,
                       "localhost:1099");
        Context context = new InitialContext(/*properties*/);
        if(args.length != 4)
        {
        	usage();
        }

        String beanName = args[0];
        
        int numberOfThreads = 0;
        try
        {
            numberOfThreads = Integer.parseInt(args[1]);
        }
        catch(NumberFormatException e)
        {
        	usage();
        }

        boolean same = true;
        if(args[2].equals("same"))
        {
        	same = true;
        }
        else if(args[2].equals("different"))
        {
        	same = false;
        }
        else
        {
        	usage();
        }
        
        int sleepTime = 0;
        try
        {
        	sleepTime = Integer.parseInt(args[3]);
        }
        catch(NumberFormatException e)
        {
        	usage();
        }

        Sleep sleep = null;
        if(same)
        {
        	sleep = (Sleep)context.lookup(beanName);
        }
        for(int i = 1; i <= numberOfThreads; i++)
        {
        	if(!same)
        	{
            	sleep = (Sleep)context.lookup(beanName);
        	}
        	SleepThread t = new SleepThread(sleep, sleepTime, i);
        	t.start();
        }
    }
    
    private static void usage()
    {
    	System.out.println("4 args required: beanName numberOfThreads [same | different] sleepTime");
    	System.exit(0);
    }
}

class SleepThread extends Thread
{
	Sleep sleep;
	int sleepTime;
	int threadNumber;

	public SleepThread(Sleep sleep, int sleepTime, int threadNumber)
	{
		this.sleep = sleep;
		this.sleepTime = sleepTime;
		this.threadNumber = threadNumber;
	}

	public void run()
	{
		System.out.println("thread number " + threadNumber + ": call sleep(" + sleepTime + ")");
		sleep.sleep(sleepTime);
		System.out.println("thread number " + threadNumber + ": return from sleep(" + sleepTime + ")");
	}
}