package javacomp.android.boundservice;

import android.os.RemoteException;
import android.util.Log;

public class CounterImpl extends Counter.Stub
{
    private int counter;

    public synchronized int increment() throws RemoteException
    {
        Log.d(this.getClass().getName(), "increment has been called (" +
                                         android.os.Process.myPid() + "-" +
                                         Thread.currentThread().getName() + ")");
        counter++;
        return counter;
    }

    public synchronized int reset() throws RemoteException
    {
        Log.d(this.getClass().getName(), "reset has been called (" +
                                         android.os.Process.myPid() + "-" +
                                         Thread.currentThread().getName() + ")");
        counter = 0;
        return counter;
    }

    public synchronized void change(Data data) throws RemoteException
    {
        Log.d(this.getClass().getName(), "change has been called (" +
                                         android.os.Process.myPid() + "-" +
                                         Thread.currentThread().getName() + ")");
        counter = counter * data.getFactor() + data.getAddend();
        data.setResult(counter);
    }

    public void multiIncrement(int times, Callback cb) throws RemoteException
    {
        Log.d(this.getClass().getName(), "callback has been called (" +
                                         android.os.Process.myPid() + "-" +
                                         Thread.currentThread().getName() + ")");
        new CallbackThread(times, this, cb).start();
    }
}

class CallbackThread extends Thread
{
    private int times;
    private CounterImpl counter;
    private Callback cb;

    public CallbackThread(int times, CounterImpl counter, Callback cb)
    {
        this.times = times;
        this.counter = counter;
        this.cb = cb;
    }

    public void run()
    {
        for (int i = 0; i < times; i++)
        {
            try
            {
                Log.d(this.getClass().getName(), "calling callback method sendMessage (" +
                                                 android.os.Process.myPid() + "-" +
                                                 Thread.currentThread().getName() + ")");
                int newValue = counter.increment();
                cb.newCounterValue(newValue);
                sleep(2000);
            }
            catch (Exception e)
            {
            }
        }
    }
}