package javacomp.android.bindservice1;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.RemoteException;
import android.util.Log;
import android.view.View;
import android.widget.TextView;

public class ServiceActivity extends Activity
{
	private Counter counter;
	private MyServiceConnection serConn;

    public void setCounter(Counter counter) 
    {
		this.counter = counter;
		TextView tv = (TextView)findViewById(R.id.textView1);
		if(counter != null)
		{
			tv.setText(R.string.msgBound);
		}
		else
		{
			tv.setText(R.string.msgUnbound);
		}
	}

	public void onCreate(Bundle savedInstanceState)
    {
        Log.d(this.getClass().getName(), "onCreate (" + android.os.Process.myPid() + "-" + Thread.currentThread().getName() + ")");
        super.onCreate(savedInstanceState);
        setContentView(R.layout.main);
        serConn = new MyServiceConnection(this);
    }

	public void onDestroy()
    {
        Log.d(this.getClass().getName(), "onDestroy (" + android.os.Process.myPid() + "-" + Thread.currentThread().getName() + ")");
		super.onDestroy();
		if(counter != null)
		{
			onUnbindService(null);
		}
    }

	public void onBindService(View v)
	{
        Log.d(this.getClass().getName(), "starting service (" + android.os.Process.myPid() + "-" + Thread.currentThread().getName() + ")");
        Intent newIntent = new Intent("de.fh_trier.example.BINDSERVICE");
        bindService(newIntent, serConn, Context.BIND_AUTO_CREATE);
	}

	public void onUnbindService(View v)
	{
        Log.d(this.getClass().getName(), "stopping service (" + android.os.Process.myPid() + "-" + Thread.currentThread().getName() + ")");
        unbindService(serConn);
        setCounter(null);
	}
	
	public void increment(View view)
	{
		action(1);
	}
	
	public void reset(View view)
	{
		action(2);
	}
	
	public void change(View view)
	{
		action(3);
	}
	
	private void action(int function)
	{
		TextView tv = (TextView)findViewById(R.id.textView2);
		if(counter != null)
		{
	        Log.d(this.getClass().getName(), "calling remote method (" + android.os.Process.myPid() + "-" + Thread.currentThread().getName() + ")");
			try
			{
				int newValue;
				switch(function)
				{
				    case 1:				    	
						newValue = counter.increment();
						break;
				    case 2:				    	
						newValue = counter.reset();
						break;
				    case 3:
				    	Data data = new Data(10, 1);
						counter.change(data);
						newValue = data.getResult();
						break;
					default:
						return;
				}
				tv.setText("" + newValue);
			}
			catch(RemoteException e)
			{
				tv.setText("Ausnahme: " + e.getMessage());
			}
		}
		else
		{
			tv.setText(R.string.msgNoOpPossible);
		}
	}
	
	public void callback(View view)
	{
		TextView tv = (TextView)findViewById(R.id.textView2);
		if(counter != null)
		{
	        Log.d(this.getClass().getName(), "calling remote method (" + android.os.Process.myPid() + "-" + Thread.currentThread().getName() + ")");
			Handler handler = new Handler();
			Callback cb = new CallbackImpl(handler, tv);
			try
			{
				counter.callbackMethod(cb);
			}
			catch(RemoteException e)
			{
				tv.setText("Ausnahme: " + e.getMessage());
			}
		}
		else
		{
			tv.setText(R.string.msgNoOpPossible);
		}
	}
	
	public void nextActivity(View view)
	{
		Intent newIntent = new Intent("de.fh_trier.example.ServiceExample2");
        startActivity(newIntent);
	}
}